==> geo-xform-1.0.4.tgz: package/lib/vrt.js <==
/* @ flow */
'use strict';

var fs = require('fs');
var Geojson = require('./geojson');
var _ = require('highland');
var EventEmitter = require('events').EventEmitter;
var util = require('util');

function createStream(options) {
  var size = options.size || 5000;

  var output = _.pipeline(function (stream) {
    var watcher = undefined;
    var first = true;
    var index = 0;
    return stream.splitBy(',{').map(filter).batch(size).consume(function (err, batch, push, next) {
      if (err) push(err);
      if (batch === _.nil) return;
      if (first) {
        watcher = new Watcher();
        watcher.on('finish', function () {
          push(null, '</OGRVRTDataSource>');
          return push(null, _.nil);
        });
        push(null, '<OGRVRTDataSource>');
        first = false;
        try {
          output.emit('properties', sample(batch));
        } catch (e) {
          output.emit('error', e);
          return output.destroy();
        }
      }
      var fileName = options.path + '/part.' + index + '.json';
      push(null, addMetadata(fileName));
      watcher.watch(writeJsonPart(batch, fileName, index));
      index++;
      next();
    });
  });
  return output;
}

function sample(batch) {
  var sample = batch.find(function (f) {
    var feature = JSON.parse(f);
    if (feature.geometry && feature.geometry.type) return true;else return false;
  });
  sample = JSON.parse(sample || batch[0]);
  var geometry = sample.geometry ? sample.geometry.type : 'NONE';
  var fields = Object.keys(sample.properties);
  return { geometry: geometry, fields: fields };
}

function filter(string) {
  var parts = string.split('"features":[{');
  return ('{' + parts[parts.length - 1]).replace(/]}}]}/, ']}}');
}

function addMetadata(fileName) {
  return '<OGRVRTLayer name="OGRGeoJSON"><SrcDataSource>' + fileName + '</SrcDataSource></OGRVRTLayer>';
}

function writeJsonPart(batch, fileName) {
  var fileStream = fs.createWriteStream(fileName);
  return _(batch).pipe(Geojson.createStream()).pipe(fileStream);
}

// This object's job is to make sure that we don't close the stream until
// all the vrt parts have been fully written to disk
function Watcher() {
  this.writers = [];
}
util.inherits(Watcher, EventEmitter);

Watcher.prototype.watch = function (writer) {
  var _this = this;

  this.writers.push(false);
  var index = this.writers.length - 1;
  writer.on('finish', function () {
    _this.writers[index] = true;
    // this will emit finish when all the outstanding writers come back
    if (_this.writers.indexOf(false) < 0) _this.emit('finish');
  });
};

module.exports = { createStream: createStream };

==> geo-xform-1.0.5.tgz: package/lib/vrt.js <==
/* @ flow */
'use strict';

var fs = require('fs');
var Geojson = require('./geojson');
var _ = require('highland');
var EventEmitter = require('events').EventEmitter;
var util = require('util');
var lodash = require('lodash');

function createStream(options) {
  var size = options.size || 5000;

  var output = _.pipeline(function (stream) {
    var watcher = new Watcher();
    var first = true;
    var index = 0;
    return stream.splitBy(',{').map(filter).batch(size).consume(function (err, batch, push, next) {
      if (err) push(err);
      if (batch === _.nil) return;
      if (first) {
        watcher.on('finish', function () {
          push(null, '</OGRVRTDataSource>');
          return push(null, _.nil);
        });
        push(null, '<OGRVRTDataSource>');
        first = false;
        try {
          output.emit('properties', sample(batch));
        } catch (e) {
          output.emit('error', e);
          return output.destroy();
        }
      }
      var fileName = options.path + '/part.' + index + '.json';
      push(null, addMetadata(fileName));
      var writer = writeLayer(batch, fileName);
      watcher.watch(writer);
      index++;
      next();
    });
  });
  return output;
}

function sample(batch) {
  var sample = lodash.find(function (f) {
    var feature = JSON.parse(f);
    if (feature.geometry && feature.geometry.type) return true;else return false;
  });
  sample = JSON.parse(sample || batch[0]);
  var geometry = sample.geometry ? sample.geometry.type : 'NONE';
  var fields = Object.keys(sample.properties);
  return { geometry: geometry, fields: fields };
}

function filter(string) {
  // strip off characters if this is the first geojson feature
  var parts = string.split('"features":[{');
  // handle the case where this is the last
  return ('{' + parts[parts.length - 1]).replace(/]}}]}/, ']}}');
}

function addMetadata(fileName) {
  return '<OGRVRTLayer name="OGRGeoJSON"><SrcDataSource>' + fileName + '</SrcDataSource></OGRVRTLayer>';
}

function writeLayer(batch, fileName) {
  var fileStream = fs.createWriteStream(fileName);
  return _(batch).pipe(Geojson.createStream()).pipe(fileStream);
}

// This object's job is to make sure that we don't close the stream until
// all the vrt parts have been fully written to disk
function Watcher() {
  this.writers = [];
}
util.inherits(Watcher, EventEmitter);

Watcher.prototype.watch = function (writer) {
  var _this = this;

  this.writers.push(false);
  var index = this.writers.length - 1;
  writer.on('finish', function () {
    _this.writers[index] = true;
    // this will emit finish when all the outstanding writers come back
    if (_this.writers.indexOf(false) < 0) _this.emit('finish');
  });
};

module.exports = { createStream: createStream };
